#ifndef _ENGINE_
#define _ENGINE_

#include <GL/gl.h>
#include <GL/glu.h>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>
#include <stdio.h>
#include <string>
#include <vector>

#include "pluginManager.h"
#include "font.h"
#include "ConfigFile.h"
#include "scrollbar.h"

using namespace std;

struct config_T
{
	int		cOVER_ANGLE;
	float 	cOVER_SPACE;	// SPACE FROM FRONT IMAGE
	float 	sPACE;	  		// SPACE BETWEEN IMAGES
	int 	cHANGE_TIME;	// MILISECONDS
	float 	zPOSITION;	  	// MAX Z POS (Image going to front)
	int		uP_ANGLE;
};

struct plugins_T			// Where it stores the plugins and its variables for the effects (cover flow + fadeIn)
{
	iGloobus*	gloobus;
	double		angle;
	double		xPos;
	double		zPos;
	double		fadeIn;				// FadeIn Value
	double		startTime;			// FadeIn start time

	double		initialPos;			// MultiCoverFlow
	double		finalPos;			// MultiCoverFlow
};

class Engine
{
	public:
		//VARIABLES
	
		bool					m_next;			// To know when to change the photo (next)
		bool					m_prev;			// To know when to change the photo (prev)
		bool					m_direct;		// When going directly to a photo
		int						m_directChangeTime;
		vector<plugins_T>		m_plugins;		// Main Plugins Vector
		int						m_actualPhoto;	// Actual Photo Number
		int						m_loadedTill;	// To know till we loaded
		Font2					m_font;			// To write the filename under the photo
		string					m_text;			// TEMPORAL
		int						m_texturesLoaded;
		int						m_window_size[2];
		
		config_T				m_config;		// Config file Manager
		pluginManager 			m_PManager;		// Plugin Manager Variable
		Scrollbar				m_scroll;		// Scrollbar Manager
		
	
		//FUNCTIONS
								Engine(int w, int h);
								~Engine();

		void 					init(string dir);
		bool 					update(void);
		void 					render(void);
		bool					draw(void);

	private:

		int						run();				// Thread that loads the files
		void					set_initial_position(int i, int direction);	// Set initial position for coverflow
		void					set_direct_position(int i, int direction);	// Set initial position for coverflow
		void					update_photo(int i,double time,int direction); // Updates the i photo position
		void					update_direct_photo(int i,double time,int direction); // Updates the i photo position
		void					init_plugins();		// Loads the textures when them are ready (It didn't work in thread)
		int						get_dir(string dir);// Load the directory and creates the pluginClasses
		void					render_text();
		void					do_fade_in(int i);	// Uses GL functions to make a neat appereance
		void					read_config();		// Reads the configuration file

};




// THREAD TRICK ========================================================= //

template <typename T>
struct SDL_ThreadProxy
{
typedef int (T::*TFunction)();
SDL_ThreadProxy(T * instance, TFunction function)
: _instance(instance)
, _function(function)
{}
static int run(SDL_ThreadProxy<T> * _this)
{
return ((_this->_instance)->*(_this->_function))();
}

private:
T * _instance;
TFunction _function;
};

template <typename T>
SDL_Thread * SDL_CreateMemberThread(T * instance, int (T::*function)())
{
SDL_ThreadProxy<T> proxy(instance, function);
typedef int (*SDL_ThreadFunction)(void *);
return SDL_CreateThread(reinterpret_cast<SDL_ThreadFunction>(SDL_ThreadProxy<T>::run), &proxy);
};
// THREAD TRICK ========================================================= //


#endif
