#include "engine.h"

#include <SDL/SDL_syswm.h>


using namespace std;


map<string, create_t *, less<string> >  factory;		//Global variable that plugin uses to register
map<string, destroy_t *, less<string> > factoryDestroy;	//Global variable that plugin uses to destroy
vector<plugin_filetype> pluginVector;					//Global variable that plugin uses to say what filetype it loads


// =========================================================================== //
void GL_Setup(int width, int height)
{
		glViewport( 0, 0, width, height );
		glMatrixMode( GL_PROJECTION );
		glEnable( GL_DEPTH_TEST );
		gluPerspective( 45, (float)width/height, 0.1, 100 );
		glMatrixMode( GL_MODELVIEW );
		//glClearColor(0,0,0,0); 									//Background color (Black)
		//glClearColor(255,255,255,0); 									//Background color (White)
		glEnable(GL_BLEND);     
}
// =========================== CENTER WINDOW ================================================= //
static void gtv_center_window(SDL_Surface *screen)
{
    SDL_SysWMinfo info;

    SDL_VERSION(&info.version);
    if ( SDL_GetWMInfo(&info) > 0 ) {
        int x, y;
        int w, h;

        if ( info.subsystem == SDL_SYSWM_X11 ) {
            info.info.x11.lock_func();
            w = DisplayWidth(info.info.x11.display,
                             DefaultScreen(info.info.x11.display));
            h = DisplayHeight(info.info.x11.display,
                             DefaultScreen(info.info.x11.display));
            x = (w - screen->w)/2;
            y = (h - screen->h)/2;
            XMoveWindow(info.info.x11.display, info.info.x11.wmwindow, x, y);
            info.info.x11.unlock_func();
        }

    }
}


// =========================================================================== //
// 									 MAIN 									   //
// =========================================================================== //
//TODO: parameter route	
int main(int argc, char* argv[])
{
	string dir="./";
	if(argc==2)
	{
		dir =argv[1];
	}

	// == READ CONFIG FILE
	ConfigFile config("/usr/share/gloobus/gloobus.config");
	int WIDTH, HEIGHT;
	config.readInto( WIDTH, "WIDTH",1000 );
	config.readInto( HEIGHT, "HEIGHT",400 );

	// == Initialize SDL with best video mode
	SDL_Init(SDL_INIT_VIDEO);
	const SDL_VideoInfo* info = SDL_GetVideoInfo();	
	int vidFlags = SDL_OPENGL | SDL_GL_DOUBLEBUFFER | SDL_HWPALETTE | SDL_RESIZABLE;
	if (info->hw_available) {vidFlags |= SDL_HWSURFACE;}
	else {vidFlags |= SDL_SWSURFACE;}
	int bpp = info->vfmt->BitsPerPixel;

	SDL_Surface * screen = SDL_SetVideoMode(WIDTH, HEIGHT, bpp, vidFlags);
	SDL_WM_SetCaption("Gloobus", NULL);		// Set window title
	gtv_center_window(screen);				// Center Screen
	GL_Setup(WIDTH, HEIGHT);				// Setup GL



	// ===== Main bucle   INIT - [ UPDATE - RENDER ] - QUIT	===== //
	Engine* engine = new Engine(WIDTH,HEIGHT);
	engine->init(dir);
	engine->render();				// RENDER FIRST TIME

	while(engine->update())	
	{
		//if(engine->draw())		// ONLY RENDER IF THERE ARE UPDATES
			{engine->render();}

	}


	delete engine;
	SDL_Quit();
}
