//g++ -fPIC -c iText.cpp
//g++ -shared -Wl,-soname,iText.so -o iText.so iText.o -lMagick++ timer.cpp

#ifndef _ITEXT_
#define _ITEXT_

#include <cmath>
#include <SDL/SDL.h>
#include <SDL/SDL_image.h>

#include <Magick++/Include.h>		// ImageMagick
#include <Magick++/Image.h>			// ImageMagick

#include "../iGloobus.h"
#include "../timer.h"

#define MANAGER_VERSION 1			// TODO: Define here the pluginManager version!!
#define PLUGIN_NAME	 	"iText"	    // TODO: Define here the plugin Name

using namespace std;
using namespace Magick;


struct Document
{
	Image			image;
	GLuint			textureID;
	float			yPos;
};

// ============================ Plugin Class iPhoto ==================================== //

class iText : public iGloobus 	//Class inherited from iGloobus
{	
	private:
	vector<Document>				m_document;
	int								m_actualPage;
	int								m_fontSize;	
	int								m_w;			// Width
	int								m_h;			// Height
	bool							m_next;			// When updating to next
	bool							m_prev;			// When changin to prev 
	
	
	public:
	virtual void load();						// Load the Photo
	virtual void init();						// Initialize the plugin
	virtual void end();							// Destroy the plugin
	virtual void render();						// Render function	
	virtual void update(char key[]);					// Update function
	virtual void get_size(int *w, int *h);		// To know the size of the GL window (on preview)
		
	void		load_texture(int i);				// Private (Load Texture)
	int			get_number_of_pages();				// Private (Get Number of pages)
	void		load_page(Document * doc,int page);	// Private Load Page;
	void		load_next_page(int direction);
	void		render_page(int i, float ratio);
	void		scale_image();						// Scale Down the page			
	
				iText();
};



// ============================ Class factories======================================== //

extern "C" iGloobus* create() {
	printf("Creating iText...\n");
    return new iText;
}

extern "C" void destroy(iGloobus* p) {
	printf("Destroying iText...\n");
    delete p;
}

extern "C" int getVersion(){			//To save crashes from older versions
	return MANAGER_VERSION;
}

// ============================== REGISTER ============================================ //
extern "C" 
{
   class proxy {
		public:
	   	proxy()
		{
			//TODO: Add here the mime filetypes that plugin supports
			plugin_filetype itself = { PLUGIN_NAME, "text/x-c; charset=us-ascii",1};		// Initial filetype

			factory[itself.name] = create;		    // Register the plugin
			factoryDestroy[itself.name] = destroy;	// Function for destroy this plugin class
			
			pluginVector.push_back(itself);			// Add filetype that plugin reads
		    itself.filetype = "text/plain; charset=us-ascii";
			pluginVector.push_back(itself);			// Add filetype that plugin reads
			itself.filetype = "text/plain; charset=utf-8";
			pluginVector.push_back(itself);			// Add filetype that plugin reads
	   }
 };
 proxy p;	// When we open the dll with RTLD_NOW it creates p, and in consequence, it registers the plugin 
}

#endif
